<?php
/**
 * Library post meta boxes handler
 *
 * @package "All In One Meta" library
 * @since   1.0.0
 * @version 1.0.0
 */

// Prevent direct script access.
if ( !defined( 'ABSPATH' ) ) {
	die( 'No direct script access allowed' );
}

if( ! class_exists( 'AIOM_Post_Metabox' ) ) {
	/**
	 * Class AIOM_Post
	 */
	final class AIOM_Post_Metabox {
		
		/**
		 * Holds meta box ID
		 * @var string|null
		 */
		private $id = NULL;
		
		/**
		 * Holds meta box title
		 * @var string
		 */
		private $title = '';
		
		/**
		 * Hold post type: post, page or any other registered post type
		 * @var array
		 */
		private $post_type = array();
		
		/**
		 * Holds meta box context. Post edit screen contexts include 'normal', 'side', and 'advanced'
		 * @var string
		 */
		private $context = 'normal';
		
		/**
		 * Holds meta box priority. high, low, default
		 * @var string
		 */
		private $priority = 'default';
		
		/**
		 * Holds fields configuration
		 * @var array
		 */
		private $fields = null;
		
		/**
		 * Holds callback that provide fields structure
		 * @var
		 */
		private $fields_callback;
		
		/**
		 * AIOM_Post_Metabox constructor.
		 *
		 * @param array $args
		 * @param string|array $fields_callback Callback that provides fields structure
		 */
		public function __construct( $args, $fields_callback ) {
			
			$this->validate_args( $args );
			$this->setup_args( $args );
			$this->setup_fields_callback( $fields_callback );
			
			$this->hooks();
		}
		
		/**
		 * Parse and validate metabox configuration
		 *
		 * @param array $args
		 *
		 * @throws Exception
		 */
		private function validate_args( &$args ) {
			$args = wp_parse_args( $args, array(
				'id'        => '',
				'title'     => '',
				'post_type' => array(),
				'context'   => 'normal',
				'priority'  => 'default',
			) );
			
			if ( ! $args[ 'id' ] ) {
				throw new Exception( __( '"id" is required', 'aiom' ) );
			}
			
			if ( ! $args[ 'title' ] ) {
				throw new Exception( __( '"title" is required', 'aiom' ) );
			}
			
			$args[ 'post_type' ] = (array)$args[ 'post_type' ];
			if ( empty( $args[ 'post_type' ] ) ) {
				throw new Exception( __( '"post_type" is required', 'aiom' ) );
			}
		}
		
		/**
		 * Setup metabox arguments
		 *
		 * @param array $args
		 */
		private function setup_args( $args ) {
			$this->id = $args[ 'id' ];
			$this->title = $args[ 'title' ];
			$this->post_type = $args[ 'post_type' ];
			$this->context = $args[ 'context' ];
			$this->priority = $args[ 'priority' ];
		}
		
		/**
		 * Setup fields callback
		 *
		 * @param string|array $callback Callback that provides fields structure
		 */
		private function setup_fields_callback( $callback ) {
			$this->fields_callback = $callback;
		}
		
		/**
		 * Get fields
		 * @return array
		 */
		private function get_fields(){
			if( null === $this->fields ) {
				$this->fields = is_callable( $this->fields_callback ) ? call_user_func( $this->fields_callback ) : array();
			}
			
			return apply_filters( 'aiom/post/fields', $this->fields, get_post() );
		}
		
		/**
		 * Setup generic actions
		 */
		private function hooks() {
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
			add_action( 'add_meta_boxes', array( $this, 'register' ), 1 );
			add_action( 'aiom_save_post', array( $this, 'save' ), 10, 3 );
		}
		
		/**
		 * Get assets version
		 * @return string
		 */
		private function get_assets_version() {
			return '1.0.0';
		}
		
		/**
		 * Get relative url
		 * @return string
		 */
		private function get_assets_url() {
			$dir = __DIR__;
			$theme_url = get_template_directory_uri();
			
			$theme_folder = substr( $theme_url, strrpos( $theme_url, '/' ) + 1 );
			$dir_url = strtr( $dir, array( '\\' => '/', DIRECTORY_SEPARATOR => '/' ) );
			
			$relative_url = rtrim( substr( $dir_url, strrpos( $dir_url, $theme_folder ) + strlen( $theme_folder ) ), 'classes' );
			
			return $theme_url . trailingslashit( untrailingslashit( $relative_url ) );
			
		}
		
		/**
		 * Enqueue Scripts and Styles
		 */
		public function admin_enqueue_scripts() {
			$screen = get_current_screen();
			
			if ( $screen && in_array( $screen->id, $this->post_type ) ) {
				$relative_url = $this->get_assets_url();
				
				wp_enqueue_style(
					'aiom-admin-meta-style',
					$relative_url . 'assets/css/aiom-metabox-style.css',
					array(),
					$this->get_assets_version()
				);
				wp_enqueue_script(
					'aiom-admin-meta-script',
					$relative_url . 'assets/js/aiom-metabox-script.js',
					array( 'jquery' ),
					$this->get_assets_version(),
					true
				);
				
				$types = array();
				foreach( $this->get_fields() as $tab_id => $tab ) {
					$types = array_merge( $types, array_values( wp_list_pluck( $tab[ 'fields' ], 'type', '' ) ) );
				}
				
				if ( ! class_exists( 'AIOM_Renderer' ) ) {
					require_once( 'class-aiom-renderer.php' );
				}
				
				foreach( $types as $type ) {
					$handler = AIOM_Renderer::get_field_handler( $type );
					
					$handler::enqueue();
				}
			}
		}
		
		/**
		 * Register meta boxes
		 */
		public function register() {
			foreach ( $this->post_type as $post_type ) {
				add_meta_box(
					$this->id,
					$this->title,
					array( $this, 'render' ),
					$post_type,
					$this->context,
					$this->priority
				);
			}
		}
		
		/**
		 * Render meta box
		 *
		 * @param WP_Post $object Current post instance
		 */
		public function render( $object ) {
			
			$meta_key = AIOM_Config::get_post_meta_key();
			$data = (array)aiom_get_post_meta( $object->ID, $meta_key );
			
			if ( ! class_exists( 'AIOM_Renderer' ) ) {
				require_once( 'class-aiom-renderer.php' );
			}
			
			$renderer = new AIOM_Renderer( $this->get_fields(), $data, $meta_key, $this->context, $object );
			$renderer->render();
		}
		
		/**
		 * Add data to data saver
		 *
		 * @param int   $post_id Post ID
		 * @param array $data    Data to save: key->value pairs
		 */
		private function add_saver_data( $post_id, $data ) {

			if ( ! class_exists( 'AIOM_Renderer' ) ) {
				require_once( 'class-aiom-renderer.php' );
			}

			$post_type = get_post_type( $post_id );

			$meta_values = array();
			$standalone_values = array();
			foreach ( $this->get_fields() as $tab_id => $tab ) {
				foreach ( $tab[ 'fields' ] as $field_id => $field ) {

					$handler = AIOM_Renderer::get_field_handler( $field[ 'type' ] );
					$field = call_user_func( array( $handler, 'parse_field_args' ), $field );

					$key = isset( $field[ 'meta_key' ] ) && $field[ 'meta_key' ] ? $field[ 'meta_key' ] : $field[ 'name' ];
					$default_value = isset( $field[ 'default' ] ) ? $field[ 'default' ] : null;
					$value = isset( $data[ $key ] ) ? $data[ $key ] : $default_value;
					$callback = isset( $field[ 'callback' ] ) ? $field[ 'callback' ] : false;
					
					if ( is_callable( $callback ) ) {
						$value = call_user_func( $callback, $value );
					}
					
					/**
					 * Provide possibility to overwrite meta value for standalone field
					 */
					$value = apply_filters( 'aiom/' . $post_type . '/save_value?field=' . $key, $value, $post_id );
					
					$meta_values[ $key ] = $value;
					
					if( isset( $field[ 'standalone' ] ) && (bool)$field[ 'standalone' ] ) {
						$standalone_values[ $key ] = $value;
					}
				}
			}
			
			/**
			 * Last chance to edit fields values
			 */
			$meta_values = apply_filters( 'aiom/' . $post_type . '/save_values', $meta_values, $post_id );
			
			$saver = AIOM_Data_Saver::get_instance();
			$saver->add_standalone_data( $standalone_values );
			$saver->add_data( $meta_values );
			
		}
		
		/**
		 * Save post meta data
		 *
		 * @param int     $post_id Current post ID
		 * @param WP_Post $post    Current Post
		 *
		 * @return bool|int
		 */
		public function save( $post_id, $post ) {
			
			if( ! wp_doing_ajax() ) {
				
				/** Add nonce for security and authentication. */
				$nonce_name = isset( $_POST[ 'aiom_nonce' ] ) ? $_POST[ 'aiom_nonce' ] : '';
				$nonce_action = 'aiom_nonce_action';
				
				/** Check if nonce is valid. */
				if ( ! wp_verify_nonce( $nonce_name, $nonce_action ) ) {
					return false;
				}
				
			}
			
			/** Check if user has permissions to save data. */
			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return false;
			}
			
			/** Check if not an auto save. */
			if ( wp_is_post_autosave( $post_id ) ) {
				return false;
			}
			
			/** Check if not a revision. */
			if ( wp_is_post_revision( $post_id ) ) {
				return false;
			}
			
			/** make sure we're working with needed post type. */
			if ( ! in_array( $post->post_type, $this->post_type ) ) {
				return false;
			}
			
			/** it's save to process */
			$meta_key = AIOM_Config::get_post_meta_key();
			$meta_data = isset( $_POST[ $meta_key ] ) ? $_POST[ $meta_key ] : array();
			
			/** Add data to saver: it must take care */
			$this->add_saver_data( $post_id, $meta_data );
			
		}
		
	}
	
}