<?php
/**
 * Library term meta boxes handler
 *
 * @package "All In One Meta" library
 * @since   1.0.0
 * @version 1.0.0
 */

// Prevent direct script access.
if ( !defined( 'ABSPATH' ) ) {
	die( 'No direct script access allowed' );
}

if( ! class_exists( 'AIOM_Taxonomy_Metabox' ) ) {
	/**
	 * Class AIOM_Taxonomy_Metabox
	 */
	final class AIOM_Taxonomy_Metabox {

		/**
		 * Holds meta box ID
		 * @var string|null
		 */
		private $id = NULL;

		/**
		 * Holds meta box title
		 * @var string
		 */
		private $title = '';

		/**
		 * Hold post type: post, page or any other registered post type
		 * @var array
		 */
		private $taxonomy = array();
		
		/**
		 * Holds fields configuration
		 * @var array
		 */
		private $fields = null;
		
		/**
		 * Holds callback that provide fields structure
		 * @var
		 */
		private $fields_callback;

		/**
		 * AIOM_Taxonomy_Metabox constructor.
		 *
		 * @param array $args
		 * @param       $fields_callback Callback that provides fields structure
		 */
		public function __construct( $args, $fields_callback ) {

			$this->validate_args( $args );
			$this->setup_args( $args );
			$this->setup_fields_callback( $fields_callback );

			$this->hooks();
		}

		/**
		 * Parse and validate metabox configuration
		 *
		 * @param array $args
		 *
		 * @throws Exception
		 */
		private function validate_args( &$args ) {
			$args = wp_parse_args( $args, array(
				'id'        => '',
				'title'     => '',
				'taxonomy' => array()
			) );

			if ( ! $args[ 'id' ] ) {
				throw new Exception( __( '"id" is required', 'aiom' ) );
			}

			if ( ! $args[ 'title' ] ) {
				throw new Exception( __( '"title" is required', 'aiom' ) );
			}

			$args[ 'taxonomy' ] = (array)$args[ 'taxonomy' ];
			if ( empty( $args[ 'taxonomy' ] ) ) {
				throw new Exception( __( '"taxonomy" is required', 'aiom' ) );
			}
		}

		/**
		 * Setup metabox arguments
		 *
		 * @param array $args
		 */
		private function setup_args( $args ) {
			$this->id = $args[ 'id' ];
			$this->title = $args[ 'title' ];
			$this->taxonomy = $args[ 'taxonomy' ];
		}
		
		/**
		 * Setup fields callback
		 *
		 * @param string|array $callback Callback that provides fields structure
		 */
		private function setup_fields_callback( $callback ) {
			$this->fields_callback = $callback;
		}
		
		/**
		 * Get fields
		 * @return array
		 */
		private function get_fields(){
			if( null === $this->fields ) {
				$this->fields = is_callable( $this->fields_callback ) ? call_user_func( $this->fields_callback ) : array();
			}
			
			$taxnow = $_REQUEST['taxonomy'];
			$tag = null;
			if( isset( $_REQUEST['tag_ID'] ) ) {
				$tag = get_term( $_REQUEST[ 'tag_ID' ], $taxnow, OBJECT, 'edit' );
			}
			
			return apply_filters( 'aiom/taxonomy/fields', $this->fields, $taxnow, $tag );
		}

		/**
		 * Setup generic actions
		 */
		private function hooks() {
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
			add_action( 'aiom_save_term', array( $this, 'save' ), 10, 2 );
			foreach( $this->taxonomy as $taxonomy ) {
				add_action( $taxonomy . '_edit_form', array( $this, 'render_edit_form' ), 10, 1 );
				add_action( $taxonomy . '_add_form_fields', array( $this, 'render_add_form' ), 10, 1 );
			}
		}

		/**
		 * Get assets version
		 * @return string
		 */
		private function get_assets_version() {
			return '1.0.0';
		}

		/**
		 * Get relative url
		 * @return string
		 */
		private function get_assets_url() {
			$dir = __DIR__;
			$theme_url = get_template_directory_uri();

			$theme_folder = substr( $theme_url, strrpos( $theme_url, '/' ) + 1 );
			$dir_url = strtr( $dir, array( '\\' => '/', DIRECTORY_SEPARATOR => '/' ) );

			$relative_url = rtrim( substr( $dir_url, strrpos( $dir_url, $theme_folder ) + strlen( $theme_folder ) ), 'classes' );

			return $theme_url . trailingslashit( untrailingslashit( $relative_url ) );

		}

		/**
		 * Enqueue Scripts and Styles
		 */
		public function admin_enqueue_scripts() {
			$screen = get_current_screen();

			if ( $screen && in_array( str_replace( 'edit-', '', $screen->id ), $this->taxonomy ) ) {
				$relative_url = $this->get_assets_url();

				wp_enqueue_style(
					'aiom-admin-meta-style',
					$relative_url . 'assets/css/aiom-metabox-style.css',
					array(),
					$this->get_assets_version()
				);
				wp_enqueue_script(
					'aiom-admin-meta-script',
					$relative_url . 'assets/js/aiom-metabox-script.js',
					array( 'jquery' ),
					$this->get_assets_version(),
					true
				);
				
				$types = array();
				foreach( $this->get_fields() as $tab_id => $tab ) {
					$types = array_merge( $types, array_values( wp_list_pluck( $tab[ 'fields' ], 'type', '' ) ) );
				}
				
				if ( ! class_exists( 'AIOM_Renderer' ) ) {
					require_once( 'class-aiom-renderer.php' );
				}
				
				foreach( $types as $type ) {
					$handler = AIOM_Renderer::get_field_handler( $type );
					
					$handler::enqueue();
				}
			}
		}

		/**
		 * Render meta box
		 * @param string $taxonomy Taxonomy slug
		 */
		public function render_add_form( $taxonomy ) {

			$meta_key = AIOM_Config::get_tax_meta_key();

			if ( ! class_exists( 'AIOM_Renderer' ) ) {
				require_once( 'class-aiom-renderer.php' );
			}

			$renderer = new AIOM_Renderer(
				$this->get_fields(),
				array(),
				$meta_key,
				'add',
				array(
					'type' => 'taxonomy',
					'taxonomy' => $taxonomy
				),
				array(
					'title' => $this->title
				)
			);
			$renderer->render();
		}

		/**
		 * Render meta box
		 * @param WP_Term $object Current Term
		 */
		public function render_edit_form( $object ) {

			$meta_key = AIOM_Config::get_tax_meta_key();
			$data = (array)aiom_get_term_meta( $object->term_id, $meta_key );

			if ( ! class_exists( 'AIOM_Renderer' ) ) {
				require_once( 'class-aiom-renderer.php' );
			}

			$renderer = new AIOM_Renderer(
				$this->get_fields(),
				$data,
				$meta_key,
				'edit',
				$object,
				array(
					'title' => $this->title
				)
			);
			$renderer->render();
		}

		/**
		 * Add data to data saver
		 *
		 * @param int    $term_id  Term ID
		 * @param string $taxonomy Taxonomy slug
		 * @param array  $data     Data to save: key->value pairs
		 */
		private function add_saver_data( $term_id, $taxonomy, $data ) {

			if ( ! class_exists( 'AIOM_Renderer' ) ) {
				require_once( 'class-aiom-renderer.php' );
			}

			$meta_values = array();
			$standalone_values = array();
			foreach ( $this->get_fields() as $tab_id => $tab ) {
				foreach ( $tab[ 'fields' ] as $field_id => $field ) {

					$handler = AIOM_Renderer::get_field_handler( $field[ 'type' ] );
					$field = call_user_func( array( $handler, 'parse_field_args' ), $field );

					$key = isset( $field[ 'meta_key' ] ) && $field[ 'meta_key' ] ? $field[ 'meta_key' ] : $field[ 'name' ];
					$default_value = isset( $field[ 'default' ] ) ? $field[ 'default' ] : null;
					$value = isset( $data[ $key ] ) ? $data[ $key ] : $default_value;
					$callback = isset( $field[ 'callback' ] ) ? $field[ 'callback' ] : false;

					if ( is_callable( $callback ) ) {
						$value = call_user_func( $callback, $value );
					}

					/**
					 * Provide possibility to overwrite meta value for standalone field
					 */
					$value = apply_filters( 'aiom/' . $taxonomy . '/save_value?field=' . $key, $value, $term_id );

					$meta_values[ $key ] = $value;

					if( isset( $field[ 'standalone' ] ) && (bool)$field[ 'standalone' ] ) {
						$standalone_values[ $key ] = $value;
					}
				}
			}

			/**
			 * Last chance to edit fields values
			 */
			$meta_values = apply_filters( 'aiom/' . $taxonomy . '/save_values', $meta_values, $term_id );

			$saver = AIOM_Data_Saver::get_instance();
			$saver->add_standalone_data( $standalone_values );
			$saver->add_data( $meta_values );

		}

		/**
		 * Save term meta data
		 *
		 * @param int    $term_id  Term ID.
		 * @param string $taxonomy Taxonomy slug.
		 *
		 * @return bool|int
		 */
		public function save( $term_id, $taxonomy ) {

			if( ! wp_doing_ajax() ) {

				/** Add nonce for security and authentication. */
				$nonce_name = isset( $_POST[ 'aiom_nonce' ] ) ? $_POST[ 'aiom_nonce' ] : '';
				$nonce_action = 'aiom_nonce_action';

				/** Check if nonce is valid. */
				if ( ! wp_verify_nonce( $nonce_name, $nonce_action ) ) {
					return false;
				}

			}

			/** Check if user has permissions to save data. */
			if ( ! current_user_can( 'edit_term', $term_id ) ) {
				return false;
			}

			/** make sure we're working with needed post type. */
			if ( ! in_array( $taxonomy, $this->taxonomy ) ) {
				return false;
			}

			/** it's save to process */
			$meta_key = AIOM_Config::get_tax_meta_key();
			$meta_data = isset( $_POST[ $meta_key ] ) ? $_POST[ $meta_key ] : array();

			/** Add data to saver: it must take care */
			$this->add_saver_data( $term_id, $taxonomy, $meta_data );
		}

	}

}