<?php
/**
 * Color Picker field for metaboxes
 *
 * @package "All In One Meta" library
 * @since   1.0.0
 * @version 1.0.0
 */

// Prevent direct script access.
if ( ! defined( 'ABSPATH' ) ) {
	die( 'No direct script access allowed' );
}

if ( ! class_exists( 'AIOM_Base_Field' ) ) {
	require_once( 'base-field.php' );
}

if ( ! class_exists( 'AIOM_Multicolor_Field' ) && class_exists( 'AIOM_Base_Field' ) ) {

	/**
	 * Class AIOM_Multicolor_Field
	 */
	final class AIOM_Multicolor_Field extends AIOM_Base_Field {

		/**
		 * Holds field choices
		 * @var array
		 */
		private $_choices;

		/**
		 * Get field choices
		 * @return array
		 */
		private function _get_choices() {
			return $this->_choices;
		}

		/**
		 * AIOM_Multicheck_Field constructor.
		 *
		 * @param array            $args
		 * @param bool|null|string $tab_id
		 * @param array            $data
		 * @param array            $structure
		 * @param string           $group
		 *
		 * @see AIOM_Base_Field::__construct()
		 */
		public function __construct( array $args, $tab_id, array $data, array $structure, $group ) {
			parent::__construct( $args, $tab_id, $data, $structure, $group );

			/***** Field choices */
			$choices = isset( $args[ 'choices' ] ) && is_array( $args[ 'choices' ] ) ? $args[ 'choices' ] : array();
			$this->_choices = $choices;
		}

		/**
		 * Parse field arguments
		 *
		 * @param array $args Field arguments
		 *
		 * @return array
		 */
		public static function parse_field_args( $args ) {
			$args = wp_parse_args( $args, array(
				'id'                 => '',
				'name'               => '',
				'meta_key'           => '',
				'choices'            => array(),
				'default'            => '',
				'label'              => '',
				'description'        => '',
				'priority'           => 0,
				'sub_priority'       => 0,
				'standalone'         => false,
				'attributes'         => '',
				'class'              => '',
				'wrapper_class'      => '',
				'wrapper_attributes' => '',
				'callback'           => array( __CLASS__, 'sanitize' ),
				'value_callback'     => null,
				'active_callback'    => array(),
			) );

			return $args;
		}
		
		/**
		 * Get field wrapper classes
		 * @return string
		 */
		public function get_wrapper_class() {
			$classes = 'aiom-form-row aiom-form-row-multicolor';
			$passed_classes = parent::get_wrapper_class();
			if( $passed_classes ) {
				$classes .= ' ' . $passed_classes;
			}
			
			return $classes;
		}

		/**
		 * Get form field attributes
		 * @return string
		 */
		protected function get_choice_attribute( $key ) {
			$values = $this->get_value();

			return $this->get_attributes() . ' data-default-color="' . $values[ $key ] . '"';
		}

		/**
		 * Enqueue color picker assets
		 */
		public static function enqueue() {
			wp_enqueue_style( 'wp-color-picker' );
			wp_enqueue_script( 'wp-color-picker' );

			$scripts = wp_scripts();
			$deps = $scripts->registered[ 'aiom-admin-meta-script' ]->deps;
			$deps[] = 'wp-color-picker';
			$scripts->registered[ 'aiom-admin-meta-script' ]->deps = array_unique( $deps );
		}

		/**
		 * Render field
		 */
		public function render() {
			$values = $this->get_value();
			$label = $this->get_label(); ?>
			<div class="<?php echo esc_attr( $this->get_wrapper_class() ); ?>" <?php echo $this->get_wrapper_attributes(); ?>>
				<div class="label-col<?php echo $label ? '' : ' label-col-empty'; ?>">
					<label for="<?php echo esc_attr( $this->get_id() ); ?>"><?php echo esc_html( $label ); ?></label>
				</div>
				<div class="control-col">
					<div class="field-list">
						<?php foreach ( $this->_get_choices() as $key => $label ) { ?>
							<div class="field-list-item">
								<label><?php echo $label; ?></label>
								<input type="text"
								       id="<?php echo esc_attr( $this->get_id() ); ?>"
								       name="<?php echo esc_attr($this->get_name() ); ?>[<?php echo esc_attr( $key ); ?>]"
								       class="<?php echo esc_attr( $this->get_class() ); ?>"
								       value="<?php echo esc_attr( $values[ $key ] ); ?>"
										<?php echo $this->get_choice_attribute( $key ); ?> />
							</div>
						<?php } ?>
					</div>
					<?php if ( $description = $this->get_description() ) { ?>
						<p class="description"><?php echo $description; ?></p>
					<?php } ?>
				</div>
				<?php echo $this->get_active_callback(); ?>
			</div>
			<?php
		}

		/**
		 * Sanitize field value
		 *
		 * @param array $values Current values
		 *
		 * @return array
		 */
		public static function sanitize( $values = array() ) {
			if ( ! is_array( $values ) ) {
				$values = (array)$values;
			}

			return $values;
		}

	}

}